<?xml version="1.0" encoding="ISO-8859-1" standalone="yes"?>
<!--
The Apache Software License, Version 1.1

Copyright (c) 1999, 2000 The Apache Software Foundation.  All rights
reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:

1. Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.

2. Redistributions in binary form must reproduce the above copyright
   notice, this list of conditions and the following disclaimer in
   the documentation and/or other materials provided with the
   distribution.

3. The end-user documentation included with the redistribution, if
   any, must include the following acknowlegement:
      "This product includes software developed by the
       Apache Software Foundation (http://www.apache.org/)."
   Alternately, this acknowlegement may appear in the software itself,
   if and wherever such third-party acknowlegements normally appear.

4. The names "The Jakarta Project", "Ant", and "Apache Software
   Foundation" must not be used to endorse or promote products derived
   from this software without prior written permission. For written
   permission, please contact apache@apache.org.

5. Products derived from this software may not be called "Apache"
   nor may "Apache" appear in their names without prior written
   permission of the Apache Group.

THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.
====================================================================

This software consists of voluntary contributions made by many
individuals on behalf of the Apache Software Foundation.  For more
information on the Apache Software Foundation, please see
<http://www.apache.org/>.
-->
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" xmlns:xrn="http://xmlreleasenotes.free.fr" version="1.0">
	<!--
	This file belongs to XMLReleaseNotes V@XRN_VERSION@.
	-->
	<!--
	The aim of this model XSLT is to generate an intermediate XML release notes file that contains the cumulative changes between two versions.
	-->
	<xsl:import href="XMLReleaseNotesCommon.xsl"/>
	<xsl:import href="XRNModelCommon.xsl"/>
	<xsl:output method="xml" indent="yes"/>
	<!--
	Enables to define the first version of the release notes from which the cumulation will be performed.
	-->
	<xsl:param name="firstVersion"/>
	<!--
	The last version of the release notes the cumulation will be performed for.
	-->
	<xsl:param name="lastVersion"/>
	<!--
	A computed variable related to the versions to be documented: see template '_printFirstVersionIndex'.
	-->
	<xsl:variable name="first_version_index">
		<xsl:for-each select="xrn:ReleaseNotes/Releases[1]">
			<xsl:call-template name="_printFirstVersionIndex"/>
		</xsl:for-each>
	</xsl:variable>
	<!--
	A computed variable related to the versions to be documented: see template '_printFirstVersionIndex'.
	-->
	<xsl:variable name="last_version_index">
		<xsl:for-each select="xrn:ReleaseNotes/Releases[1]">
			<xsl:call-template name="_printLastVersionIndex"/>
		</xsl:for-each>
	</xsl:variable>
	<!--
	Outputs the exact name of the first version from which the documentation should be generated.
	The current node must be the 'ReleaseNotes/Releases' XML element.
	-->
	<xsl:template name="_printFirstVersionName">
		<xsl:choose>
			<xsl:when test="$firstVersion">
				<xsl:value-of select="$firstVersion"/>
			</xsl:when>
			<xsl:otherwise>
				<xsl:value-of select="Release[last()]/@version"/>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>
	<!--
	Same idea as the '_printFirstVersionName' template.
	-->
	<xsl:template name="_printLastVersionName">
		<xsl:choose>
			<xsl:when test="$lastVersion">
				<xsl:value-of select="$lastVersion"/>
			</xsl:when>
			<xsl:otherwise>
				<xsl:value-of select="Release[1]/@version"/>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>
	<!--
	Outputs the index (position) of the first version from which the documentation should be generated.
	The current node must be the 'ReleaseNotes/Releases' XML element.
	-->
	<xsl:template name="_printFirstVersionIndex">
		<xsl:variable name="actual_first_version">
			<xsl:call-template name="_printFirstVersionName"/>
		</xsl:variable>
		<xsl:choose>
			<xsl:when test="count(Release[@version = $actual_first_version]) > 0">
				<xsl:for-each select="Release">
					<xsl:if test="@version = $actual_first_version">
						<xsl:value-of select="position()"/>
					</xsl:if>
				</xsl:for-each>
			</xsl:when>
			<xsl:otherwise>
				<xsl:message>Warning: the release with version '<xsl:value-of select="$firstVersion"/>' does not exist: the release notes for component with ID '<xsl:value-of select="../@ID"/>' will be generated from the first version (at the bottom)</xsl:message>
				<xsl:value-of select="count(Release)"/>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>
	<!--
	Same idea as the '_printFirstVersionIndex' template.
	-->
	<xsl:template name="_printLastVersionIndex">
		<xsl:variable name="actual_last_version">
			<xsl:call-template name="_printLastVersionName"/>
		</xsl:variable>
		<xsl:choose>
			<xsl:when test="count(Release[@version = $actual_last_version]) > 0">
				<xsl:for-each select="Release">
					<xsl:if test="@version = $actual_last_version">
						<xsl:value-of select="position()"/>
					</xsl:if>
				</xsl:for-each>
			</xsl:when>
			<xsl:otherwise>
				<xsl:message>Warning: the release with version '<xsl:value-of select="$lastVersion"/>' does not exist: the release notes for component with ID '<xsl:value-of select="../@ID"/>' will be generated until the last version (at the top)</xsl:message>
				<xsl:text>1</xsl:text>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>
	<xsl:template match="xrn:ReleaseNotes/Releases/Release">
		<xsl:choose>
			<xsl:when test="count(preceding-sibling::Release) = ($last_version_index - 1)">
				<xsl:copy>
					<xsl:attribute name="version"><xsl:for-each select=".."><xsl:call-template name="_printFirstVersionName"/><xsl:text>-</xsl:text><xsl:call-template name="_printLastVersionName"/></xsl:for-each></xsl:attribute>
					<xsl:copy-of select="@*[name() != 'version']"/>
					<xsl:apply-templates/>
				</xsl:copy>
			</xsl:when>
			<xsl:otherwise>
				<xsl:copy-of select="."/>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>
	<xsl:template match="xrn:ReleaseNotes/Releases/Release/Features">
		<xsl:choose>
			<xsl:when test="count(../preceding-sibling::Release) = ($last_version_index - 1)">
				<xsl:copy>
					<xsl:copy-of select="@*"/>
					<xsl:apply-templates/>
					<xsl:for-each select="../../Release[position() &gt; $last_version_index and position() &lt;= $first_version_index ]">
						<xsl:copy-of select="Features/Feature"/>
					</xsl:for-each>
				</xsl:copy>
			</xsl:when>
			<xsl:otherwise>
				<xsl:copy-of select="."/>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>
</xsl:stylesheet>
